@@ -59,7 +59,7 @@ int conf_blreset_on_talk = 0;
 list_t *parse_conf(FILE *file);
 static void conf_die(char *fmt, ...);
 #ifdef HAVE_LIBSSL
-static int adm_trust(struct link_client *ic, struct line *line);
+int adm_trust(struct link_client *ic, struct line *line);
 #endif
 
 static void hash_binary(char *hex, unsigned char **password, unsigned int *seed)
@@ -70,7 +70,7 @@ static void hash_binary(char *hex, unsigned char **password, unsigned int *seed)
 
 	if (strlen(hex) != 40)
 		fatal("Incorrect password format %s\n", hex);
-	
+
 	md5 = malloc(20);
 	for (i = 0; i < 20; i++) {
 		sscanf(hex + 2 * i, "%02x", &buf);
@@ -245,8 +245,8 @@ pid_is_there:
 		}
 		if (pid)
 			mylog(LOG_INFO, "pid file found (pid %ld).", pid);
-		mylog(LOG_INFO, "Another instance of bip is certainly runing.");
-		mylog(LOG_INFO, "If you are sure this is not the case remove"
+		mylog(LOG_STD, "Another instance of bip is certainly runing.");
+		mylog(LOG_STD, "If you are sure this is not the case remove"
 					" %s.", conf_pid_file);
 		exit(2);
 	}
@@ -327,7 +327,7 @@ static int add_network(list_t *data)
 				return 0;
 			break;
 		default:
-			conf_die("uknown keyword in network statement");
+			conf_die("unknown keyword in network statement");
 			if (t->type == TUPLE_STR)
 				free(t->pdata);
 			return 0;
@@ -365,7 +365,7 @@ static int add_channel(list_t *channell, list_t *data)
 			c->key = t->pdata;
 			break;
 		default:
-			conf_die("uknown keyword in channel statement");
+			conf_die("unknown keyword in channel statement");
 			if (t->type == TUPLE_STR)
 				free(t->pdata);
 			return 0;
@@ -436,6 +436,9 @@ static int add_connection(list_t *connectionl, list_t *data,
 		case LEX_PASSWORD:
 			c->password = t->pdata;
 			break;
+		case LEX_VHOST:
+			c->vhost = t->pdata;
+			break;
 		case LEX_CHANNEL:
 			r = add_channel(&c->channell, t->pdata);
 			free(t->pdata);
@@ -455,7 +458,7 @@ static int add_connection(list_t *connectionl, list_t *data,
 			c->on_connect_send = t->pdata;
 			break;
 		default:
-			conf_die("uknown keyword in connection statement");
+			conf_die("unknown keyword in connection statement");
 			if (t->type == TUPLE_STR)
 				free(t->pdata);
 			return 0;
@@ -526,7 +529,7 @@ static int add_user(list_t *data)
 				r = add_connection(&u->connectionl, t->pdata,
 						NULL);
 			else
-				r = add_connection(&u->connectionl, t->pdata, 
+				r = add_connection(&u->connectionl, t->pdata,
 						&old_u->connectionl);
 			free(t->pdata);
 			if (!r)
@@ -674,8 +677,10 @@ int fireup(FILE *conf)
 
 	if (!conf_biphome) {
 		char *home = getenv("HOME");
-		if (!home)
+		if (!home) {
 			conf_die("no $HOME !, do you live in a trailer ?");
+			return 0;
+		}
 		conf_biphome = malloc(strlen(home) + strlen("/.bip") + 1);
 		strcpy(conf_biphome, home);
 		strcat(conf_biphome, "/.bip");
@@ -769,7 +774,6 @@ void ircize(list_t *ll)
 					struct c_channel *chan =
 						list_it_item(&chit);
 					struct chan_info *ci = chan_info_new();
-					printf("%s\n",chan->name);
 					ci->name = strdup(chan->name);
 					ci->key = strmaydup(chan->key);
 					hash_insert(&link->chan_infos,
@@ -798,7 +802,7 @@ void ircize(list_t *ll)
 				MAYFREE(link->ssl_check_store);
 				sk_X509_free(link->untrusted_certs);
 #endif
-				
+
 
 				for (i = 0; i < link->serverc; i++)
 					server_free(link->serverv[i]);
@@ -891,7 +895,7 @@ int main(int argc, char **argv)
 
 	conf_log_root = NULL;
 	conf_log_format = NULL;
-	conf_log_level = LOG_ERROR;
+	conf_log_level = LOG_INFO;
 	conf_backlog = 1;
 	conf_log = 1;
 	conf_backlog_lines = 100;
@@ -1076,7 +1080,7 @@ static int ssl_discard_next_cert(struct link_client *ic)
 #endif
 
 #ifdef HAVE_LIBSSL
-static int adm_trust(struct link_client *ic, struct line *line)
+int adm_trust(struct link_client *ic, struct line *line)
 {
 	if (ic->allow_trust != 1) {
 		mylog(LOG_ERROR, "User attempted TRUST command without "
@@ -1100,7 +1104,7 @@ static int adm_trust(struct link_client *ic, struct line *line)
 		/* OK, attempt to trust the cert! */
 		BIO *bio = BIO_new_file(LINK(ic)->ssl_check_store, "a+");
 		X509 *trustcert = sk_X509_shift(LINK(ic)->untrusted_certs);
-		
+
 		if(!bio || !trustcert ||
 				PEM_write_bio_X509(bio, trustcert) <= 0)
 			write_line_fast(CONN(ic), ":irc.bip.net NOTICE pouet "
@@ -1160,7 +1164,7 @@ int adm_bip(struct link_client *ic, struct line *line)
 		nick = LINK(ic)->prev_nick;
 	if (line->elemc < 2)
 		return OK_FORGET;
-	
+
 	if (strcasecmp(line->elemv[1], "RELOAD") == 0) {
 		reloading_client = ic;
 		sighup = 1;
@@ -1193,7 +1197,7 @@ void free_conf(list_t *l)
 		switch (t->tuple_type) {
 		case TUPLE_STR:
 			free(t->pdata);	/* no break, for the style */
-		case TUPLE_INT:	
+		case TUPLE_INT:
 			free(t);
 			break;
 		case TUPLE_LIST:
