@@ -165,10 +165,10 @@ list_t *channel_name_list(struct channel *c)
 			strncat(str, " ", NAMESIZE);
 			len++;
 		}
-		if (n->ovmask == NICKOP) {
+		if (n->ovmask & NICKOP) {
 			strncat(str, "@", NAMESIZE);
 			len++;
-		} else if (n->ovmask == NICKVOICED) {
+		} else if (n->ovmask & NICKVOICED) {
 			strncat(str, "+", NAMESIZE);
 			len++;
 		}
@@ -309,6 +309,16 @@ static void irc_server_connected(struct link_server *server)
         }
 }
 
+static int who_arg_to_ovmask(char *str)
+{
+	int ovmask = 0;
+	if (strchr(str, '@'))
+		ovmask |= NICKOP;
+	if (strchr(str, '+'))
+		ovmask |= NICKVOICED;
+	return ovmask;
+}
+
 /*
  * Given the way irc nets disrespect the rfc, maybe we should completely forget
  * about this damn ircmask...
@@ -317,22 +327,54 @@ static void irc_server_connected(struct link_server *server)
 static int parse_352(struct link_server *server, struct line *line)
 {
 	char *im;
-	int ret = OK_FORGET;
-	if (line->elemc < 4)
-		return ERR_PROTOCOL;
-	if (strcmp(line->elemv[1], server->nick) != 0)
-		return ret;
 
-	im = malloc(strlen(server->nick) + strlen(line->elemv[3]) +
-			strlen(line->elemv[4]) + 2 + 1);
-	strcpy(im, server->nick);
-	strcat(im, "!");
-	strcat(im, line->elemv[3]);
-	strcat(im, "@");
-	strcat(im, line->elemv[4]);
-	set_ircmask(server, im);
-	free(im);
-	return ret;
+	if (line->elemc < 8)
+		return ERR_PROTOCOL;
+
+	if (strcmp(line->elemv[2], "*") == 0) {
+		if (server->irc_mask &&
+				strcmp(line->elemv[1], server->nick) == 0) {
+			im = malloc(strlen(server->nick) +
+					strlen(line->elemv[3]) +
+					strlen(line->elemv[4]) + 2 + 1);
+			strcpy(im, server->nick);
+			strcat(im, "!");
+			strcat(im, line->elemv[3]);
+			strcat(im, "@");
+			strcat(im, line->elemv[4]);
+			set_ircmask(server, im);
+			free(im);
+			return OK_FORGET;
+		}
+	} else {
+		struct channel *channel;
+		struct nick *nick;
+
+		channel = hash_get(&server->channels, line->elemv[2]);
+		if (!channel)
+			return OK_COPY;
+
+		nick = hash_get(&channel->nicks, line->elemv[6]);
+		if (!nick)
+			return OK_COPY;
+
+		nick->ovmask = who_arg_to_ovmask(line->elemv[7]);
+	}
+	return OK_COPY_WHO;
+}
+
+static int irc_315(struct link_server *server, struct line *l)
+{
+	struct link *link = LINK(server);
+	if (link->who_client) {
+		--link->who_client->who_count;
+
+		if (link->who_client->who_count < 0)
+			fatal("negative who count");
+	}
+	l = NULL; /* keep gcc happy */
+
+	return OK_COPY_WHO;
 }
 
 /*
@@ -346,10 +388,6 @@ int irc_dispatch_server(struct link_server *server, struct line *line)
 	if (line->elemc == 0)
 		return ERR_PROTOCOL;
 
-	if (!server->irc_mask &&
-			strcmp(line->elemv[0], "352") == 0) /* WHO RPL */
-		ret = parse_352(server, line);
-
 	if (strcmp(line->elemv[0], "PING") == 0) {
 		if (line->elemc < 2)
 			return ERR_PROTOCOL;
@@ -396,10 +434,18 @@ int irc_dispatch_server(struct link_server *server, struct line *line)
 		ret = OK_FORGET;
 		if (strcmp(line->elemv[0], "376") == 0) /* end of motd */
 			irc_server_connected(server);
+		else if (strcmp(line->elemv[0], "422") == 0) /* no motd */
+				irc_server_connected(server);
+
 	} else if (LINK(server)->s_state == IRCS_CONNECTING) {
 		ret = OK_FORGET;
 		if (strcmp(line->elemv[0], "NOTICE") == 0) {
 		} else if (strcmp(line->elemv[0], "376") == 0) {
+							/* end of motd */
+			irc_server_connected(server);
+			list_add_last(&LINK(server)->init_strings,
+					irc_line_dup(line));
+		} else if (strcmp(line->elemv[0], "422") == 0) { /* no motd */
 			irc_server_connected(server);
 			list_add_last(&LINK(server)->init_strings,
 					irc_line_dup(line));
@@ -421,6 +467,10 @@ int irc_dispatch_server(struct link_server *server, struct line *line)
 		ret = irc_332(server, line);
 	} else if (strcmp(line->elemv[0], "333") == 0) {
 		ret = irc_333(server, line);
+	} else if (strcmp(line->elemv[0], "352") == 0) {
+		ret = parse_352(server, line);
+	} else if (strcmp(line->elemv[0], "315") == 0) {
+		ret = irc_315(server, line);
 	} else if (strcmp(line->elemv[0], "353") == 0) {
 		ret = irc_353(server, line);
 	} else if (strcmp(line->elemv[0], "366") == 0) {
@@ -451,6 +501,39 @@ int irc_dispatch_server(struct link_server *server, struct line *line)
 			free(s);
 		}
 	}
+	if (ret == OK_COPY_WHO && LINK(server)->who_client) {
+		char *s;
+
+		s = irc_line_to_string(line);
+		write_line(CONN(LINK(server)->who_client), s);
+		free(s);
+
+		if (LINK(server)->who_client->who_count == 0) {
+			int i;
+			for (i = 0; i < LINK(server)->l_clientc; i++) {
+				struct link_client *ic =
+						LINK(server)->l_clientv[i];
+				if (ic == LINK(server)->who_client) {
+					if (!list_is_empty(&ic->who_queue))
+						fatal("who active and queued?");
+					continue;
+				}
+
+				if (!list_is_empty(&ic->who_queue)) {
+					char *l;
+					while ((l = list_remove_first(
+							&ic->who_queue))) {
+						write_line(CONN(server), l);
+						free(l);
+					}
+					LINK(server)->who_client = ic;
+					break;
+				}
+			}
+			if (i == LINK(server)->l_clientc)
+				LINK(server)->who_client = NULL;
+		}
+	}
 	return ret;
 }
 
@@ -523,11 +606,16 @@ void unbind_from_link(struct link_client *ic)
 {
 	struct link *l = LINK(ic);
 	int i;
+
 	for (i = 0; i < l->l_clientc; i++)
 		if (l->l_clientv[i] == ic)
 			break;
 	if (i == l->l_clientc)
 		fatal("unbind_from_link");
+
+	if (l->who_client == ic)
+		l->who_client = NULL;
+
 	for (i = i + 1; i < l->l_clientc; i++)
 		l->l_clientv[i - 1] = l->l_clientv[i];
 
@@ -676,12 +764,13 @@ static int irc_cli_startup(struct link_client *ic, struct line *line,
 	if (LINK(ic)->s_state != IRCS_CONNECTED) {
 		/* Check if we have an untrusted certificate from the server */
 		if (ssl_check_trust(ic)) {
+			TYPE(ic) = IRC_TYPE_TRUST_CLIENT;
 			ic->allow_trust = 1;
 			free(init_nick);
 			return OK_FORGET;
 		}
 	}
-#endif	
+#endif
 
 	if (LINK(ic)->s_state == IRCS_NONE) {
 		/* drop it if corresponding server hasn't connected at all. */
@@ -756,7 +845,7 @@ static int irc_cli_nick(struct link_client *ic, struct line *line, list_t *cl)
 
 	if ((ic->state & IRCC_PASS) != IRCC_PASS)
 		WRITE_LINE2(CONN(ic), P_SERV, "NOTICE", ic->init_nick,
-				"You should type /QUOTE PASS your_password");
+				"You should type /QUOTE PASS your_username:your_password:your_connection_name");
 
 	return OK_FORGET;
 }
@@ -815,6 +904,23 @@ static int irc_cli_notice(struct link_client *ic, struct line *line)
 	return OK_COPY_CLI;
 }
 
+static int irc_cli_who(struct link_client *ic, struct line *line)
+{
+	struct link *l = LINK(ic);
+
+	if (l->who_client && l->who_client != ic) {
+		list_add_first(&ic->who_queue, irc_line_to_string(line));
+		return OK_FORGET;
+	}
+
+	if (!l->who_client)
+		l->who_client = ic;
+
+	++ic->who_count;
+
+	return OK_COPY;
+}
+
 #if 0
 static void unbind_to_server(struct link_client *ic)
 {
@@ -959,6 +1065,21 @@ static int irc_cli_part(struct link_client *irc, struct line *line)
 	return OK_COPY;
 }
 
+#ifdef HAVE_LIBSSL
+static int irc_dispatch_trust_client(struct link_client *ic, struct line *line) 
+{
+	int r = OK_COPY;
+	if (line->elemc < 2)
+		return ERR_PROTOCOL;
+
+	if (strcmp(line->elemv[0], "BIP") == 0 &&
+	    strcmp(line->elemv[1], "TRUST") == 0)
+		r = adm_trust(ic, line);
+	
+	return r;
+}
+#endif
+
 int irc_cli_bip(struct link_client *ic, struct line *line);
 static int irc_dispatch_client(struct link_client *ic, struct line *line)
 {
@@ -990,6 +1111,8 @@ static int irc_dispatch_client(struct link_client *ic, struct line *line)
 		r = irc_cli_privmsg(ic, line);
 	} else if (strcmp(line->elemv[0], "NOTICE") == 0) {
 		r = irc_cli_notice(ic, line);
+	} else if (strcmp(line->elemv[0], "WHO") == 0) {
+		r = irc_cli_who(ic, line);
 	}
 
 	if (r == OK_COPY || r == OK_COPY_CLI) {
@@ -1094,6 +1217,11 @@ int irc_dispatch(struct link_any *l, struct line *line, list_t *linkl)
 		return irc_dispatch_loging_client((struct link_client*)l,
 				line, linkl);
 		break;
+#ifdef HAVE_LIBSSL		
+	case IRC_TYPE_TRUST_CLIENT:
+		return irc_dispatch_trust_client((struct link_client*)l, line);
+		break;
+#endif
 	default:
 		fatal("gn");
 	}
@@ -1482,12 +1610,12 @@ static int irc_mode(struct link_server *server, struct line *line)
 			}
 			break;
 		case 'l':
-			if (add) {
+			if (add)
 				cur_arg++;
-			}
 			break;
 		case 'h':
 		case 'H':
+		case 'e':
 			if (cur_arg + 3 >= line->elemc)
 				return ERR_PROTOCOL;
 			cur_arg++;
@@ -1809,6 +1937,16 @@ static void irc_close(struct link_any *l)
 	}
 }
 
+struct link_client *irc_client_new(void)
+{
+	struct link_client *c;
+
+	c = calloc(sizeof(struct link_client), 1);
+	list_init(&c->who_queue, list_ptr_cmp);
+
+	return c;
+}
+
 struct link_server *irc_server_new(struct link *link, connection_t *conn)
 {
 	struct link_server *s;
@@ -1844,6 +1982,10 @@ connection_t *irc_server_connect(struct link *link)
 	struct link_server *ls;
 	connection_t *conn;
 
+	mylog(LOG_INFO, "Connecting user '%s' to network '%s' using server "
+		"%s:%d", link->username, link->name,
+		link->serverv[link->cur_server]->host,
+		link->serverv[link->cur_server]->port);
 	conn = connection_new(link->serverv[link->cur_server]->host,
 				link->serverv[link->cur_server]->port,
 				link->vhost, link->bind_port,
